/**
 *
 */
package de.intevation.gnv.geobackend.sde.datasources;

import com.vividsolutions.jts.geom.Coordinate;
import com.vividsolutions.jts.geom.Envelope;

import java.io.Serializable;


import org.apache.log4j.Logger;

/**
 * @author Tim Englich (tim.englich@intevation.de)
 * @author Sascha L. Teichmann (sascha.teichmann@intevation.de)
 */
public class RasterObject 
implements   Serializable
{
    private static Logger log = Logger.getLogger(RasterObject.class);

    public static final int NEAREST_NEIGHBOR = 0;
    public static final int BILINEAR         = 1;

    private double mx;
    private double bx;
    private double my;
    private double by;

    private int columnIndex;
    private int rowIndex;

    private int tileWidth;
    private int tileHeight;

    private double [] rasterData;

    public RasterObject() {
    }

    public RasterObject(
        double mx, double bx,
        double my, double by,
        int       columnIndex, 
        int       rowIndex,
        double [] rasterData,
        int       tileWidth,
        int       tileHeight
    ){
        this.mx          = mx;
        this.bx          = bx;
        this.my          = my;
        this.by          = by;
        this.columnIndex = columnIndex;
        this.rowIndex    = rowIndex;
        this.rasterData  = rasterData;
        this.tileWidth   = tileWidth;
        this.tileHeight  = tileHeight;
    }

    public boolean contains(Coordinate coordinate) {
        double px = mx*coordinate.x + bx;
        double py = my*coordinate.y + by;
        return px >= 0d && py >= 0d && px < tileWidth && py < tileHeight;
    }

    public final double get(int posX, int posY) {
        return rasterData[posY*tileWidth + posX];
    }

    public int getTileWidth() {
        return tileWidth;
    }

    public int getTileHeight() {
        return tileWidth;
    }

    public int getColumnIndex() {
        return columnIndex;
    }

    public int getRowIndex() {
        return rowIndex;
    }
    
    public double getValue(Coordinate coordinate) {
        return getValue(coordinate, NEAREST_NEIGHBOR);
    }

    public double getValue(Coordinate coordinate, int interpolationType) {

        double px = mx*coordinate.x + bx;
        double py = my*coordinate.y + by;

        if (px < 0d || py < 0d || px >= tileWidth || py >= tileHeight) {
            return Double.NaN;
        }

        /*
        if (interpolationType == BILINEAR) {
            // TODO: implement me!
        }
        */

        int posX = Math.min(tileWidth-1,  (int)Math.round(px));
        int posY = Math.min(tileHeight-1, (int)Math.round(py));

        return get(posX, posY);
    }
}
